@@ -1,7 +1,9 @@
 import datetime
 import enum
+import inspect
 import secrets
 import time
+from contextlib import ContextDecorator
 from typing import Any, Dict, Optional
 
 import torch
@@ -237,7 +239,7 @@ class Timer:
         """Start/resume the timer.
 
         If the timer is on pause, the method resumes the timer.
-        If the timer is runnning, the method does nothing (i.e. it does NOT overwrite
+        If the timer is running, the method does nothing (i.e. it does NOT overwrite
         the previous pause time).
         """
         if self._start_time is None:
@@ -383,25 +385,27 @@ class Timer:
         self.__dict__.update(state)
 
 
-class evaluation(torch.no_grad):
+class evaluation(ContextDecorator):
     """Context-manager & decorator for models evaluation.
 
     This code... ::
 
-        with evaluation(model):
+        with evaluation(model):  # or: evaluation(model_0, model_1, ...)
             ...
 
-        @evaluation(model)
+        @evaluation(model)  # or: @evaluation(model_0, model_1, ...)
         def f():
             ...
 
     ...is equivalent to the following: ::
 
-        with torch.no_grad():
+        context = getattr(torch, 'inference_mode', torch.no_grad)
+
+        with context():
             model.eval()
             ...
 
-        @torch.no_grad()
+        @context()
         def f():
             model.eval()
             ...
@@ -414,9 +418,19 @@ class evaluation(torch.no_grad):
         decorated function returns.
 
     Warning:
-        The function must be used in the same way as `torch.no_grad`, i.e. only as a
-        context manager or a decorator as shown below in the examples. Otherwise, the
-        behaviour is undefined.
+        The function must be used in the same way as `torch.no_grad` and
+        `torch.inference_mode`, i.e. only as a context manager or a decorator as shown
+        below in the examples. Otherwise, the behaviour is undefined.
+
+    Warning:
+        Contrary to `torch.no_grad` and `torch.inference_mode`, the function cannot be
+        used to decorate generators. So, in the case of generators, you have to manually
+        create a context::
+
+            def my_generator():
+                with evaluation(...):
+                    for a in b:
+                        yield c
 
     Examples:
         .. testcode::
@@ -435,34 +449,43 @@ class evaluation(torch.no_grad):
             @evaluation(a, b)
             def f():
                 ...
-
-        .. testcode::
-
-            model = torch.nn.Linear(1, 1)
-            for grad_before_context in False, True:
-                for train in False, True:
-                    torch.set_grad_enabled(grad_before_context)
-                    model.train(train)
-                    with evaluation(model):
-                        assert not model.training
-                        assert not torch.is_grad_enabled()
-                        ...
-                    assert torch.is_grad_enabled() == grad_before_context
-                    # model.training is unspecified here
     """
 
     def __init__(self, *modules: nn.Module) -> None:
         assert modules
         self._modules = modules
+        self._torch_context = None
+
+    def __call__(self, func):
+        """Decorate a function with an evaluation context.
+
+        Args:
+            func
+        Raises:
+            AssertionError: if :code:`func` is a generator
+        """
+        assert not inspect.isgeneratorfunction(
+            func
+        ), f'{self.__class__} cannot be used to decorate generators. See the documentation.'
+        return super().__call__(func)
 
     def __enter__(self) -> None:
-        result = super().__enter__()
+        assert self._torch_context is None
+        self._torch_context = getattr(torch, 'inference_mode', torch.no_grad)()
+        self._torch_context.__enter__()  # type: ignore
         for m in self._modules:
             m.eval()
+
+    def __exit__(self, *exc):
+        assert self._torch_context is not None
+        result = self._torch_context.__exit__(*exc)  # type: ignore
+        self._torch_context = None
         return result
 
 
-def improve_reproducibility(seed: Optional[int]) -> int:
+def improve_reproducibility(
+    base_seed: Optional[int], one_cuda_seed: bool = False
+) -> int:
     """Set seeds in `random`, `numpy` and `torch` and make some cuDNN operations deterministic.
 
     Do everything possible to improve reproducibility for code that relies on global
@@ -475,16 +498,16 @@ def improve_reproducibility(seed: Optional[int]) -> int:
     3. `torch.backends.cudnn.deterministic` to `True`
 
     Args:
-        seed: the seed for all mentioned libraries. Must be a non-negative number less
-            than :code:`2 ** 32 - 1`. If `None`, a high-quality seed is generated
-            instead.
+        base_seed: the argument for `zero.random.seed`. If `None`, a high-quality base
+            seed is generated instead.
+        one_cuda_seed: the argument for `zero.random.seed`.
 
     Returns:
-        seed: if :code:`seed` is set to `None`, the generated seed is returned;
-            otherwise, :code:`seed` is returned as is
+        base_seed: if :code:`base_seed` is set to `None`, the generated base seed is
+            returned; otherwise, :code:`base_seed` is returned as is
 
     Note:
-        If you don't want to set the seed by hand, but still want to have a chance to
+        If you don't want to choose the base seed, but still want to have a chance to
         reproduce things, you can use the following pattern::
 
             print('Seed:', zero.improve_reproducibility(None))
@@ -502,10 +525,10 @@ def improve_reproducibility(seed: Optional[int]) -> int:
     """
     torch.backends.cudnn.benchmark = False  # type: ignore
     torch.backends.cudnn.deterministic = True  # type: ignore
-    if seed is None:
+    if base_seed is None:
         # See https://numpy.org/doc/1.18/reference/random/bit_generators/index.html#seeding-and-entropy  # noqa
-        seed = secrets.randbits(128) % (2 ** 32 - 1)
+        base_seed = secrets.randbits(128) % (2 ** 32 - 1024)
     else:
-        assert seed < (2 ** 32 - 1)
-    zero_random.seed(seed)
-    return seed
+        assert base_seed < (2 ** 32 - 1024)
+    zero_random.seed(base_seed, one_cuda_seed)
+    return base_seed
