@@ -59,7 +59,7 @@ class Enumerate(Dataset):
 
     def __len__(self) -> int:
         """Get the length of the underlying dataset."""
-        return len(self._dataset)
+        return len(self._dataset)  # type: ignore
 
     def __getitem__(self, index) -> Tuple[Any, Any]:
         """Return index and the corresponding item from the underlying dataset.
@@ -302,7 +302,7 @@ class IndexLoader:
             device: if not CPU, then all indices are materialized and moved to the
                 device at the beginning of every loop. It can be useful when the indices
                 are applied to non-CPU data (e.g. CUDA-tensors) and moving data between
-                devices takes non-negligable time (which can happen in the case of
+                devices takes non-negligible time (which can happen in the case of
                 simple and fast models like MLPs).
             **kwargs: keyword arguments for `torch.utils.data.DataLoader`
         Raises:
@@ -366,15 +366,15 @@ def iter_batches(
     Returns:
         Iterator over batches.
 
+    Note:
+        If you want to infinitely iterate over batches, wrap the function in
+        :code:`while True:`.
+
     Warning:
         Numpy-arrays are not supported because of how they behave when indexed by a
         torch tensor of the size 1. For details, see
         `the issue <https://github.com/numpy/numpy/issues/16543>`_
 
-    Note:
-        If you want to infititely iterate over batches, wrap the function in
-        :code:`while True:`.
-
     See also:
         - `zero.data.IndexLoader`
         - `concat`
